package net.knarcraft.knarlib.util;

import net.knarcraft.knarlib.property.ColorConversion;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.Color;
import org.jetbrains.annotations.NotNull;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A helper class for dealing with colors
 */
@SuppressWarnings("unused")
public final class ColorHelper {

    private static boolean requireAmpersandInHexColors = false;

    private ColorHelper() {

    }

    /**
     * Inverts the given color
     *
     * @param color <p>The color to invert</p>
     * @return <p>The inverted color</p>
     */
    public static @NotNull Color invert(@NotNull Color color) {
        return color.setRed(255 - color.getRed()).setGreen(255 - color.getGreen()).setBlue(255 - color.getBlue());
    }

    /**
     * Gets the chat color corresponding to the given color
     *
     * @param color <p>The color to convert into a chat color</p>
     * @return <p>The resulting chat color</p>
     */
    public static @NotNull ChatColor fromColor(@NotNull Color color) {
        return ChatColor.of(String.format("#%02X%02X%02X", color.getRed(), color.getGreen(), color.getBlue()));
    }

    /**
     * Sets whether to require ampersand in hex color
     *
     * <p>If set to true, &amp;#f35336 will be treated as a color code, but #f35336 won't. By default, this is set to false,
     * meaning that both are treated as color codes.</p>
     *
     * @param requireAmpersandInHexColors <p>True if hex colors should require an ampersand</p>
     */
    public static void setRequireAmpersandInHexColors(boolean requireAmpersandInHexColors) {
        ColorHelper.requireAmpersandInHexColors = requireAmpersandInHexColors;
    }

    /**
     * Strips all color codes from the given message
     *
     * @param message         <p>The message to strip color codes from</p>
     * @param colorConversion <p>The type of color conversion to apply before stripping</p>
     * @return <p>The message without color codes</p>
     */
    public static @NotNull String stripColorCodes(@NotNull String message, @NotNull ColorConversion colorConversion) {
        return ChatColor.stripColor(translateColorCodes(message, colorConversion));
    }

    /**
     * Translates color codes according to the given color conversion setting
     *
     * @param message         <p>The message to translate color codes for</p>
     * @param colorConversion <p>The type of color conversion to apply</p>
     * @return <p>The string with color codes applied</p>
     */
    public static @NotNull String translateColorCodes(@NotNull String message, @NotNull ColorConversion colorConversion) {
        return switch (colorConversion) {
            case NONE -> message;
            case NORMAL -> ChatColor.translateAlternateColorCodes('&', message);
            case RGB -> translateAllColorCodes(message, false);
            case RGB_ONLY -> translateAllColorCodes(message, true);
        };
    }

    /**
     * Translates all found color codes to formatting in a string
     *
     * @param message <p>The string to search for color codes</p>
     * @param onlyRGB <p>Whether to only convert RGB (hexadecimal) color codes. If false, &[a-f0-9] will be converted as well.</p>
     * @return <p>The message with color codes translated</p>
     */
    private static @NotNull String translateAllColorCodes(@NotNull String message, boolean onlyRGB) {
        if (!onlyRGB) {
            message = ChatColor.translateAlternateColorCodes('&', message);
        }
        Pattern pattern;
        if (requireAmpersandInHexColors) {
            pattern = Pattern.compile("&(#[a-fA-F0-9]{6})");
        } else {
            pattern = Pattern.compile("&?(#[a-fA-F0-9]{6})");
        }
        Matcher matcher = pattern.matcher(message);
        while (matcher.find()) {
            message = message.replace(matcher.group(), "" + ChatColor.of(matcher.group(1)));
        }
        return message;
    }

}
